@@ -1,13 +1,23 @@
+from importlib import import_module
+
 import numpy as np
+
 import pandas as pd
 from pandas.util import testing as tm
 
+for imp in ['pandas.util', 'pandas.tools.hashing']:
+    try:
+        hashing = import_module(imp)
+        break
+    except:
+        pass
 
-class algorithm(object):
+class Algorithms(object):
     goal_time = 0.2
 
     def setup(self):
         N = 100000
+        np.random.seed(1234)
 
         self.int_unique = pd.Int64Index(np.arange(N * 5))
         # cache is_unique
@@ -17,28 +27,42 @@ class algorithm(object):
         self.float = pd.Float64Index(np.random.randn(N).repeat(5))
 
         # Convenience naming.
-        self.checked_add = pd.core.nanops._checked_add_with_arr
+        self.checked_add = pd.core.algorithms.checked_add_with_arr
 
         self.arr = np.arange(1000000)
         self.arrpos = np.arange(1000000)
         self.arrneg = np.arange(-1000000, 0)
         self.arrmixed = np.array([1, -1]).repeat(500000)
+        self.strings = tm.makeStringIndex(100000)
 
-    def time_int_factorize(self):
+        self.arr_nan = np.random.choice([True, False], size=1000000)
+        self.arrmixed_nan = np.random.choice([True, False], size=1000000)
+
+        # match
+        self.uniques = tm.makeStringIndex(1000).values
+        self.all = self.uniques.repeat(10)
+
+    def time_factorize_string(self):
+        self.strings.factorize()
+
+    def time_factorize_int(self):
         self.int.factorize()
 
-    def time_float_factorize(self):
+    def time_factorize_float(self):
         self.int.factorize()
 
-    def time_int_unique_duplicated(self):
+    def time_duplicated_int_unique(self):
         self.int_unique.duplicated()
 
-    def time_int_duplicated(self):
+    def time_duplicated_int(self):
         self.int.duplicated()
 
-    def time_float_duplicated(self):
+    def time_duplicated_float(self):
         self.float.duplicated()
 
+    def time_match_strings(self):
+        pd.match(self.all, self.uniques)
+
     def time_add_overflow_pos_scalar(self):
         self.checked_add(self.arr, 1)
 
@@ -57,8 +81,18 @@ class algorithm(object):
     def time_add_overflow_mixed_arr(self):
         self.checked_add(self.arr, self.arrmixed)
 
+    def time_add_overflow_first_arg_nan(self):
+        self.checked_add(self.arr, self.arrmixed, arr_mask=self.arr_nan)
 
-class hashing(object):
+    def time_add_overflow_second_arg_nan(self):
+        self.checked_add(self.arr, self.arrmixed, b_mask=self.arrmixed_nan)
+
+    def time_add_overflow_both_arg_nan(self):
+        self.checked_add(self.arr, self.arrmixed, arr_mask=self.arr_nan,
+                         b_mask=self.arrmixed_nan)
+
+
+class Hashing(object):
     goal_time = 0.2
 
     def setup(self):
@@ -78,13 +112,13 @@ class hashing(object):
         self.df.iloc[10:20] = np.nan
 
     def time_frame(self):
-        self.df.hash()
+        hashing.hash_pandas_object(self.df)
 
     def time_series_int(self):
-        self.df.E.hash()
+        hashing.hash_pandas_object(self.df.E)
 
     def time_series_string(self):
-        self.df.B.hash()
+        hashing.hash_pandas_object(self.df.B)
 
     def time_series_categorical(self):
-        self.df.C.hash()
+        hashing.hash_pandas_object(self.df.C)
