@@ -1,9 +1,12 @@
 from .pandas_vb_common import *
 import pandas as pd
-import pandas.computation.expressions as expr
+try:
+    import pandas.core.computation.expressions as expr
+except ImportError:
+    import pandas.computation.expressions as expr
 
 
-class eval_frame(object):
+class Eval(object):
     goal_time = 0.2
 
     params = [['numexpr', 'python'], [1, 'all']]
@@ -34,8 +37,11 @@ class eval_frame(object):
         df, df2, df3, df4 = self.df, self.df2, self.df3, self.df4
         pd.eval('df * df2 * df3 * df4', engine=engine)
 
+    def teardown(self, engine, threads):
+        expr.set_numexpr_threads()
 
-class query_datetime_index(object):
+
+class Query(object):
     goal_time = 0.2
 
     def setup(self):
@@ -45,41 +51,19 @@ class query_datetime_index(object):
         self.s = Series(self.index)
         self.ts = self.s.iloc[self.halfway]
         self.df = DataFrame({'a': np.random.randn(self.N), }, index=self.index)
+        self.df2 = DataFrame({'dates': self.s.values,})
+
+        self.df3 = DataFrame({'a': np.random.randn(self.N),})
+        self.min_val = self.df3['a'].min()
+        self.max_val = self.df3['a'].max()
 
     def time_query_datetime_index(self):
         ts = self.ts
         self.df.query('index < @ts')
 
-
-class query_datetime_series(object):
-    goal_time = 0.2
-
-    def setup(self):
-        self.N = 1000000
-        self.halfway = ((self.N // 2) - 1)
-        self.index = date_range('20010101', periods=self.N, freq='T')
-        self.s = Series(self.index)
-        self.ts = self.s.iloc[self.halfway]
-        self.df = DataFrame({'dates': self.s.values, })
-
     def time_query_datetime_series(self):
         ts = self.ts
-        self.df.query('dates < @ts')
-
-
-class query_with_boolean_selection(object):
-    goal_time = 0.2
-
-    def setup(self):
-        self.N = 1000000
-        self.halfway = ((self.N // 2) - 1)
-        self.index = date_range('20010101', periods=self.N, freq='T')
-        self.s = Series(self.index)
-        self.ts = self.s.iloc[self.halfway]
-        self.N = 1000000
-        self.df = DataFrame({'a': np.random.randn(self.N), })
-        self.min_val = self.df['a'].min()
-        self.max_val = self.df['a'].max()
+        self.df2.query('dates < @ts')
 
     def time_query_with_boolean_selection(self):
         min_val, max_val = self.min_val, self.max_val
