@@ -1,28 +1,32 @@
 from pandas import *
 import pandas as pd
-from datetime import timedelta
 from numpy.random import randn
 from numpy.random import randint
-from numpy.random import permutation
 import pandas.util.testing as tm
 import random
 import numpy as np
 import threading
+from importlib import import_module
+
 try:
     from pandas.compat import range
 except ImportError:
     pass
 
 np.random.seed(1234)
-try:
-    import pandas._tseries as lib
-except:
-    import pandas.lib as lib
+
+# try em until it works!
+for imp in ['pandas._libs.lib', 'pandas.lib', 'pandas_tseries']:
+    try:
+        lib = import_module(imp)
+        break
+    except:
+        pass
 
 try:
-    Panel = WidePanel
+    Panel = Panel
 except Exception:
-    pass
+    Panel = WidePanel
 
 # didn't add to namespace until later
 try:
