@@ -12,16 +12,24 @@ if [ -n "$LOCALE_OVERRIDE" ]; then
     python -c "$pycmd"
 fi
 
+if [ "$SLOW" ]; then
+    TEST_ARGS="--only-slow --skip-network"
+fi
+
 if [ "$BUILD_TEST" ]; then
     echo "We are not running pytest as this is a build test."
+
 elif [ "$DOC" ]; then
     echo "We are not running pytest as this is a doc-build"
+
 elif [ "$COVERAGE" ]; then
     echo pytest -s -m "single" --cov=pandas --cov-report xml:/tmp/cov-single.xml --junitxml=/tmp/single.xml $TEST_ARGS pandas
     pytest -s -m "single" --cov=pandas --cov-report xml:/tmp/cov-single.xml --junitxml=/tmp/single.xml $TEST_ARGS pandas
+
 else
     echo pytest -m "single" -r xX --junitxml=/tmp/single.xml $TEST_ARGS pandas
     pytest -m "single" -r xX  --junitxml=/tmp/single.xml $TEST_ARGS pandas # TODO: doctest
+
 fi
 
 RET="$?"
