@@ -1,12 +1,15 @@
 # -*- coding: utf-8 -*-
 
+import pytest
+
+from warnings import catch_warnings
 import numpy as np
 from pandas import Series, DataFrame, Index, Float64Index
 from pandas.util.testing import assert_series_equal, assert_almost_equal
 import pandas.util.testing as tm
 
 
-class TestFloatIndexers(tm.TestCase):
+class TestFloatIndexers(object):
 
     def check(self, result, original, indexer, getitem):
         """
@@ -45,13 +48,13 @@ class TestFloatIndexers(tm.TestCase):
 
             def f():
                 s.iloc[3.0]
-            self.assertRaisesRegexp(TypeError,
-                                    'cannot do positional indexing',
-                                    f)
+            tm.assert_raises_regex(TypeError,
+                                   'cannot do positional indexing',
+                                   f)
 
             def f():
                 s.iloc[3.0] = 0
-            self.assertRaises(TypeError, f)
+            pytest.raises(TypeError, f)
 
     def test_scalar_non_numeric(self):
 
@@ -77,7 +80,8 @@ class TestFloatIndexers(tm.TestCase):
                                       (lambda x: x, True)]:
 
                     def f():
-                        idxr(s)[3.0]
+                        with catch_warnings(record=True):
+                            idxr(s)[3.0]
 
                     # gettitem on a DataFrame is a KeyError as it is indexing
                     # via labels on the columns
@@ -85,7 +89,7 @@ class TestFloatIndexers(tm.TestCase):
                         error = KeyError
                     else:
                         error = TypeError
-                    self.assertRaises(error, f)
+                    pytest.raises(error, f)
 
                 # label based can be a TypeError or KeyError
                 def f():
@@ -95,15 +99,15 @@ class TestFloatIndexers(tm.TestCase):
                     error = KeyError
                 else:
                     error = TypeError
-                self.assertRaises(error, f)
+                pytest.raises(error, f)
 
                 # contains
-                self.assertFalse(3.0 in s)
+                assert 3.0 not in s
 
                 # setting with a float fails with iloc
                 def f():
                     s.iloc[3.0] = 0
-                self.assertRaises(TypeError, f)
+                pytest.raises(TypeError, f)
 
                 # setting with an indexer
                 if s.index.inferred_type in ['categorical']:
@@ -119,25 +123,26 @@ class TestFloatIndexers(tm.TestCase):
                     #    s2 = s.copy()
                     #    def f():
                     #        idxr(s2)[3.0] = 0
-                    #    self.assertRaises(TypeError, f)
+                    #    pytest.raises(TypeError, f)
                     pass
 
                 else:
 
                     s2 = s.copy()
                     s2.loc[3.0] = 10
-                    self.assertTrue(s2.index.is_object())
+                    assert s2.index.is_object()
 
                     for idxr in [lambda x: x.ix,
                                  lambda x: x]:
                         s2 = s.copy()
-                        idxr(s2)[3.0] = 0
-                        self.assertTrue(s2.index.is_object())
+                        with catch_warnings(record=True):
+                            idxr(s2)[3.0] = 0
+                        assert s2.index.is_object()
 
             # fallsback to position selection, series only
             s = Series(np.arange(len(i)), index=i)
             s[3]
-            self.assertRaises(TypeError, lambda: s[3.0])
+            pytest.raises(TypeError, lambda: s[3.0])
 
     def test_scalar_with_mixed(self):
 
@@ -151,15 +156,16 @@ class TestFloatIndexers(tm.TestCase):
                      lambda x: x.iloc]:
 
             def f():
-                idxr(s2)[1.0]
+                with catch_warnings(record=True):
+                    idxr(s2)[1.0]
 
-            self.assertRaises(TypeError, f)
+            pytest.raises(TypeError, f)
 
-        self.assertRaises(KeyError, lambda: s2.loc[1.0])
+        pytest.raises(KeyError, lambda: s2.loc[1.0])
 
         result = s2.loc['b']
         expected = 2
-        self.assertEqual(result, expected)
+        assert result == expected
 
         # mixed index so we have label
         # indexing
@@ -167,20 +173,21 @@ class TestFloatIndexers(tm.TestCase):
                      lambda x: x]:
 
             def f():
-                idxr(s3)[1.0]
+                with catch_warnings(record=True):
+                    idxr(s3)[1.0]
 
-            self.assertRaises(TypeError, f)
+            pytest.raises(TypeError, f)
 
             result = idxr(s3)[1]
             expected = 2
-            self.assertEqual(result, expected)
+            assert result == expected
 
-        self.assertRaises(TypeError, lambda: s3.iloc[1.0])
-        self.assertRaises(KeyError, lambda: s3.loc[1.0])
+        pytest.raises(TypeError, lambda: s3.iloc[1.0])
+        pytest.raises(KeyError, lambda: s3.loc[1.0])
 
         result = s3.loc[1.5]
         expected = 3
-        self.assertEqual(result, expected)
+        assert result == expected
 
     def test_scalar_integer(self):
 
@@ -199,7 +206,8 @@ class TestFloatIndexers(tm.TestCase):
                                       (lambda x: x.loc, False),
                                       (lambda x: x, True)]:
 
-                    result = idxr(s)[3.0]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[3.0]
                     self.check(result, s, 3, getitem)
 
                 # coerce to equal int
@@ -208,7 +216,8 @@ class TestFloatIndexers(tm.TestCase):
                                       (lambda x: x, True)]:
 
                     if isinstance(s, Series):
-                        compare = self.assertEqual
+                        def compare(x, y):
+                            assert x == y
                         expected = 100
                     else:
                         compare = tm.assert_series_equal
@@ -220,17 +229,18 @@ class TestFloatIndexers(tm.TestCase):
                                               index=range(len(s)), name=3)
 
                     s2 = s.copy()
-                    idxr(s2)[3.0] = 100
+                    with catch_warnings(record=True):
+                        idxr(s2)[3.0] = 100
 
-                    result = idxr(s2)[3.0]
-                    compare(result, expected)
+                        result = idxr(s2)[3.0]
+                        compare(result, expected)
 
-                    result = idxr(s2)[3]
-                    compare(result, expected)
+                        result = idxr(s2)[3]
+                        compare(result, expected)
 
                 # contains
                 # coerce to equal int
-                self.assertTrue(3.0 in s)
+                assert 3.0 in s
 
     def test_scalar_float(self):
 
@@ -247,22 +257,26 @@ class TestFloatIndexers(tm.TestCase):
                                   (lambda x: x, True)]:
 
                 # getting
-                result = idxr(s)[indexer]
+                with catch_warnings(record=True):
+                    result = idxr(s)[indexer]
                 self.check(result, s, 3, getitem)
 
                 # setting
                 s2 = s.copy()
 
                 def f():
-                    idxr(s2)[indexer] = expected
-                result = idxr(s2)[indexer]
+                    with catch_warnings(record=True):
+                        idxr(s2)[indexer] = expected
+                with catch_warnings(record=True):
+                    result = idxr(s2)[indexer]
                 self.check(result, s, 3, getitem)
 
                 # random integer is a KeyError
-                self.assertRaises(KeyError, lambda: idxr(s)[3.5])
+                with catch_warnings(record=True):
+                    pytest.raises(KeyError, lambda: idxr(s)[3.5])
 
             # contains
-            self.assertTrue(3.0 in s)
+            assert 3.0 in s
 
             # iloc succeeds with an integer
             expected = s.iloc[3]
@@ -273,11 +287,11 @@ class TestFloatIndexers(tm.TestCase):
             self.check(result, s, 3, False)
 
             # iloc raises with a float
-            self.assertRaises(TypeError, lambda: s.iloc[3.0])
+            pytest.raises(TypeError, lambda: s.iloc[3.0])
 
             def g():
                 s2.iloc[3.0] = 0
-            self.assertRaises(TypeError, g)
+            pytest.raises(TypeError, g)
 
     def test_slice_non_numeric(self):
 
@@ -300,7 +314,7 @@ class TestFloatIndexers(tm.TestCase):
 
                     def f():
                         s.iloc[l]
-                    self.assertRaises(TypeError, f)
+                    pytest.raises(TypeError, f)
 
                     for idxr in [lambda x: x.ix,
                                  lambda x: x.loc,
@@ -308,8 +322,9 @@ class TestFloatIndexers(tm.TestCase):
                                  lambda x: x]:
 
                         def f():
-                            idxr(s)[l]
-                        self.assertRaises(TypeError, f)
+                            with catch_warnings(record=True):
+                                idxr(s)[l]
+                        pytest.raises(TypeError, f)
 
                 # setitem
                 for l in [slice(3.0, 4),
@@ -318,15 +333,16 @@ class TestFloatIndexers(tm.TestCase):
 
                     def f():
                         s.iloc[l] = 0
-                    self.assertRaises(TypeError, f)
+                    pytest.raises(TypeError, f)
 
                     for idxr in [lambda x: x.ix,
                                  lambda x: x.loc,
                                  lambda x: x.iloc,
                                  lambda x: x]:
                         def f():
-                            idxr(s)[l] = 0
-                        self.assertRaises(TypeError, f)
+                            with catch_warnings(record=True):
+                                idxr(s)[l] = 0
+                        pytest.raises(TypeError, f)
 
     def test_slice_integer(self):
 
@@ -349,7 +365,8 @@ class TestFloatIndexers(tm.TestCase):
                 for idxr in [lambda x: x.loc,
                              lambda x: x.ix]:
 
-                    result = idxr(s)[l]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[l]
 
                     # these are all label indexing
                     # except getitem which is positional
@@ -364,7 +381,7 @@ class TestFloatIndexers(tm.TestCase):
                 def f():
                     s[l]
 
-                self.assertRaises(TypeError, f)
+                pytest.raises(TypeError, f)
 
             # getitem out-of-bounds
             for l in [slice(-6, 6),
@@ -372,7 +389,8 @@ class TestFloatIndexers(tm.TestCase):
 
                 for idxr in [lambda x: x.loc,
                              lambda x: x.ix]:
-                    result = idxr(s)[l]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[l]
 
                     # these are all label indexing
                     # except getitem which is positional
@@ -387,7 +405,7 @@ class TestFloatIndexers(tm.TestCase):
             def f():
                 s[slice(-6.0, 6.0)]
 
-            self.assertRaises(TypeError, f)
+            pytest.raises(TypeError, f)
 
             # getitem odd floats
             for l, res1 in [(slice(2.5, 4), slice(3, 5)),
@@ -397,7 +415,8 @@ class TestFloatIndexers(tm.TestCase):
                 for idxr in [lambda x: x.loc,
                              lambda x: x.ix]:
 
-                    result = idxr(s)[l]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[l]
                     if oob:
                         res = slice(0, 0)
                     else:
@@ -409,7 +428,7 @@ class TestFloatIndexers(tm.TestCase):
                 def f():
                     s[l]
 
-                self.assertRaises(TypeError, f)
+                pytest.raises(TypeError, f)
 
             # setitem
             for l in [slice(3.0, 4),
@@ -419,15 +438,16 @@ class TestFloatIndexers(tm.TestCase):
                 for idxr in [lambda x: x.loc,
                              lambda x: x.ix]:
                     sc = s.copy()
-                    idxr(sc)[l] = 0
-                    result = idxr(sc)[l].values.ravel()
-                    self.assertTrue((result == 0).all())
+                    with catch_warnings(record=True):
+                        idxr(sc)[l] = 0
+                        result = idxr(sc)[l].values.ravel()
+                    assert (result == 0).all()
 
                 # positional indexing
                 def f():
                     s[l] = 0
 
-                self.assertRaises(TypeError, f)
+                pytest.raises(TypeError, f)
 
     def test_integer_positional_indexing(self):
         """ make sure that we are raising on positional indexing
@@ -449,7 +469,7 @@ class TestFloatIndexers(tm.TestCase):
                 def f():
                     idxr(s)[l]
 
-                self.assertRaises(TypeError, f)
+                pytest.raises(TypeError, f)
 
     def test_slice_integer_frame_getitem(self):
 
@@ -467,7 +487,8 @@ class TestFloatIndexers(tm.TestCase):
                           slice(0, 1.0),
                           slice(0.0, 1.0)]:
 
-                    result = idxr(s)[l]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[l]
                     indexer = slice(0, 2)
                     self.check(result, s, indexer, False)
 
@@ -475,7 +496,7 @@ class TestFloatIndexers(tm.TestCase):
                     def f():
                         s[l]
 
-                    self.assertRaises(TypeError, f)
+                    pytest.raises(TypeError, f)
 
                 # getitem out-of-bounds
                 for l in [slice(-10, 10),
@@ -488,21 +509,22 @@ class TestFloatIndexers(tm.TestCase):
                 def f():
                     s[slice(-10.0, 10.0)]
 
-                self.assertRaises(TypeError, f)
+                pytest.raises(TypeError, f)
 
                 # getitem odd floats
                 for l, res in [(slice(0.5, 1), slice(1, 2)),
                                (slice(0, 0.5), slice(0, 1)),
                                (slice(0.5, 1.5), slice(1, 2))]:
 
-                    result = idxr(s)[l]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[l]
                     self.check(result, s, res, False)
 
                     # positional indexing
                     def f():
                         s[l]
 
-                    self.assertRaises(TypeError, f)
+                    pytest.raises(TypeError, f)
 
                 # setitem
                 for l in [slice(3.0, 4),
@@ -510,15 +532,16 @@ class TestFloatIndexers(tm.TestCase):
                           slice(3.0, 4.0)]:
 
                     sc = s.copy()
-                    idxr(sc)[l] = 0
-                    result = idxr(sc)[l].values.ravel()
-                    self.assertTrue((result == 0).all())
+                    with catch_warnings(record=True):
+                        idxr(sc)[l] = 0
+                        result = idxr(sc)[l].values.ravel()
+                    assert (result == 0).all()
 
                     # positional indexing
                     def f():
                         s[l] = 0
 
-                    self.assertRaises(TypeError, f)
+                    pytest.raises(TypeError, f)
 
     def test_slice_float(self):
 
@@ -537,25 +560,27 @@ class TestFloatIndexers(tm.TestCase):
                              lambda x: x]:
 
                     # getitem
-                    result = idxr(s)[l]
+                    with catch_warnings(record=True):
+                        result = idxr(s)[l]
                     if isinstance(s, Series):
-                        self.assert_series_equal(result, expected)
+                        tm.assert_series_equal(result, expected)
                     else:
-                        self.assert_frame_equal(result, expected)
+                        tm.assert_frame_equal(result, expected)
                     # setitem
                     s2 = s.copy()
-                    idxr(s2)[l] = 0
-                    result = idxr(s2)[l].values.ravel()
-                    self.assertTrue((result == 0).all())
+                    with catch_warnings(record=True):
+                        idxr(s2)[l] = 0
+                        result = idxr(s2)[l].values.ravel()
+                    assert (result == 0).all()
 
     def test_floating_index_doc_example(self):
 
         index = Index([1.5, 2, 3, 4.5, 5])
         s = Series(range(5), index=index)
-        self.assertEqual(s[3], 2)
-        self.assertEqual(s.ix[3], 2)
-        self.assertEqual(s.loc[3], 2)
-        self.assertEqual(s.iloc[3], 3)
+        assert s[3] == 2
+        assert s.loc[3] == 2
+        assert s.loc[3] == 2
+        assert s.iloc[3] == 3
 
     def test_floating_misc(self):
 
@@ -565,7 +590,7 @@ class TestFloatIndexers(tm.TestCase):
 
         # label based slicing
         result1 = s[1.0:3.0]
-        result2 = s.ix[1.0:3.0]
+        result2 = s.loc[1.0:3.0]
         result3 = s.loc[1.0:3.0]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, result3)
@@ -573,24 +598,24 @@ class TestFloatIndexers(tm.TestCase):
         # exact indexing when found
         result1 = s[5.0]
         result2 = s.loc[5.0]
-        result3 = s.ix[5.0]
-        self.assertEqual(result1, result2)
-        self.assertEqual(result1, result3)
+        result3 = s.loc[5.0]
+        assert result1 == result2
+        assert result1 == result3
 
         result1 = s[5]
         result2 = s.loc[5]
-        result3 = s.ix[5]
-        self.assertEqual(result1, result2)
-        self.assertEqual(result1, result3)
+        result3 = s.loc[5]
+        assert result1 == result2
+        assert result1 == result3
 
-        self.assertEqual(s[5.0], s[5])
+        assert s[5.0] == s[5]
 
         # value not found (and no fallbacking at all)
 
         # scalar integers
-        self.assertRaises(KeyError, lambda: s.loc[4])
-        self.assertRaises(KeyError, lambda: s.ix[4])
-        self.assertRaises(KeyError, lambda: s[4])
+        pytest.raises(KeyError, lambda: s.loc[4])
+        pytest.raises(KeyError, lambda: s.loc[4])
+        pytest.raises(KeyError, lambda: s[4])
 
         # fancy floats/integers create the correct entry (as nan)
         # fancy tests
@@ -598,13 +623,13 @@ class TestFloatIndexers(tm.TestCase):
         for fancy_idx in [[5.0, 0.0], np.array([5.0, 0.0])]:  # float
             assert_series_equal(s[fancy_idx], expected)
             assert_series_equal(s.loc[fancy_idx], expected)
-            assert_series_equal(s.ix[fancy_idx], expected)
+            assert_series_equal(s.loc[fancy_idx], expected)
 
         expected = Series([2, 0], index=Index([5, 0], dtype='int64'))
         for fancy_idx in [[5, 0], np.array([5, 0])]:  # int
             assert_series_equal(s[fancy_idx], expected)
             assert_series_equal(s.loc[fancy_idx], expected)
-            assert_series_equal(s.ix[fancy_idx], expected)
+            assert_series_equal(s.loc[fancy_idx], expected)
 
         # all should return the same as we are slicing 'the same'
         result1 = s.loc[2:5]
@@ -624,17 +649,17 @@ class TestFloatIndexers(tm.TestCase):
         assert_series_equal(result1, result3)
         assert_series_equal(result1, result4)
 
-        result1 = s.ix[2:5]
-        result2 = s.ix[2.0:5.0]
-        result3 = s.ix[2.0:5]
-        result4 = s.ix[2.1:5]
+        result1 = s.loc[2:5]
+        result2 = s.loc[2.0:5.0]
+        result3 = s.loc[2.0:5]
+        result4 = s.loc[2.1:5]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, result3)
         assert_series_equal(result1, result4)
 
         # combined test
         result1 = s.loc[2:5]
-        result2 = s.ix[2:5]
+        result2 = s.loc[2:5]
         result3 = s[2:5]
 
         assert_series_equal(result1, result2)
@@ -643,7 +668,7 @@ class TestFloatIndexers(tm.TestCase):
         # list selection
         result1 = s[[0.0, 5, 10]]
         result2 = s.loc[[0.0, 5, 10]]
-        result3 = s.ix[[0.0, 5, 10]]
+        result3 = s.loc[[0.0, 5, 10]]
         result4 = s.iloc[[0, 2, 4]]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, result3)
@@ -651,14 +676,14 @@ class TestFloatIndexers(tm.TestCase):
 
         result1 = s[[1.6, 5, 10]]
         result2 = s.loc[[1.6, 5, 10]]
-        result3 = s.ix[[1.6, 5, 10]]
+        result3 = s.loc[[1.6, 5, 10]]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, result3)
         assert_series_equal(result1, Series(
             [np.nan, 2, 4], index=[1.6, 5, 10]))
 
         result1 = s[[0, 1, 2]]
-        result2 = s.ix[[0, 1, 2]]
+        result2 = s.loc[[0, 1, 2]]
         result3 = s.loc[[0, 1, 2]]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, result3)
@@ -666,24 +691,183 @@ class TestFloatIndexers(tm.TestCase):
             [0.0, np.nan, np.nan], index=[0, 1, 2]))
 
         result1 = s.loc[[2.5, 5]]
-        result2 = s.ix[[2.5, 5]]
+        result2 = s.loc[[2.5, 5]]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, Series([1, 2], index=[2.5, 5.0]))
 
         result1 = s[[2.5]]
-        result2 = s.ix[[2.5]]
+        result2 = s.loc[[2.5]]
         result3 = s.loc[[2.5]]
         assert_series_equal(result1, result2)
         assert_series_equal(result1, result3)
         assert_series_equal(result1, Series([1], index=[2.5]))
 
     def test_floating_tuples(self):
-        # GH13509
+        # see gh-13509
         s = Series([(1, 1), (2, 2), (3, 3)], index=[0.0, 0.1, 0.2], name='foo')
-        result = s[0.0]
-        self.assertEqual(result, (1, 1))
 
-        s = Series([(1, 1), (2, 2), (3, 3)], index=[0.0, 0.0, 0.2], name='foo')
         result = s[0.0]
+        assert result == (1, 1)
+
         expected = Series([(1, 1), (2, 2)], index=[0.0, 0.0], name='foo')
-        assert_series_equal(result, expected)
+        s = Series([(1, 1), (2, 2), (3, 3)], index=[0.0, 0.0, 0.2], name='foo')
+
+        result = s[0.0]
+        tm.assert_series_equal(result, expected)
+
+    def test_float64index_slicing_bug(self):
+        # GH 5557, related to slicing a float index
+        ser = {256: 2321.0,
+               1: 78.0,
+               2: 2716.0,
+               3: 0.0,
+               4: 369.0,
+               5: 0.0,
+               6: 269.0,
+               7: 0.0,
+               8: 0.0,
+               9: 0.0,
+               10: 3536.0,
+               11: 0.0,
+               12: 24.0,
+               13: 0.0,
+               14: 931.0,
+               15: 0.0,
+               16: 101.0,
+               17: 78.0,
+               18: 9643.0,
+               19: 0.0,
+               20: 0.0,
+               21: 0.0,
+               22: 63761.0,
+               23: 0.0,
+               24: 446.0,
+               25: 0.0,
+               26: 34773.0,
+               27: 0.0,
+               28: 729.0,
+               29: 78.0,
+               30: 0.0,
+               31: 0.0,
+               32: 3374.0,
+               33: 0.0,
+               34: 1391.0,
+               35: 0.0,
+               36: 361.0,
+               37: 0.0,
+               38: 61808.0,
+               39: 0.0,
+               40: 0.0,
+               41: 0.0,
+               42: 6677.0,
+               43: 0.0,
+               44: 802.0,
+               45: 0.0,
+               46: 2691.0,
+               47: 0.0,
+               48: 3582.0,
+               49: 0.0,
+               50: 734.0,
+               51: 0.0,
+               52: 627.0,
+               53: 70.0,
+               54: 2584.0,
+               55: 0.0,
+               56: 324.0,
+               57: 0.0,
+               58: 605.0,
+               59: 0.0,
+               60: 0.0,
+               61: 0.0,
+               62: 3989.0,
+               63: 10.0,
+               64: 42.0,
+               65: 0.0,
+               66: 904.0,
+               67: 0.0,
+               68: 88.0,
+               69: 70.0,
+               70: 8172.0,
+               71: 0.0,
+               72: 0.0,
+               73: 0.0,
+               74: 64902.0,
+               75: 0.0,
+               76: 347.0,
+               77: 0.0,
+               78: 36605.0,
+               79: 0.0,
+               80: 379.0,
+               81: 70.0,
+               82: 0.0,
+               83: 0.0,
+               84: 3001.0,
+               85: 0.0,
+               86: 1630.0,
+               87: 7.0,
+               88: 364.0,
+               89: 0.0,
+               90: 67404.0,
+               91: 9.0,
+               92: 0.0,
+               93: 0.0,
+               94: 7685.0,
+               95: 0.0,
+               96: 1017.0,
+               97: 0.0,
+               98: 2831.0,
+               99: 0.0,
+               100: 2963.0,
+               101: 0.0,
+               102: 854.0,
+               103: 0.0,
+               104: 0.0,
+               105: 0.0,
+               106: 0.0,
+               107: 0.0,
+               108: 0.0,
+               109: 0.0,
+               110: 0.0,
+               111: 0.0,
+               112: 0.0,
+               113: 0.0,
+               114: 0.0,
+               115: 0.0,
+               116: 0.0,
+               117: 0.0,
+               118: 0.0,
+               119: 0.0,
+               120: 0.0,
+               121: 0.0,
+               122: 0.0,
+               123: 0.0,
+               124: 0.0,
+               125: 0.0,
+               126: 67744.0,
+               127: 22.0,
+               128: 264.0,
+               129: 0.0,
+               260: 197.0,
+               268: 0.0,
+               265: 0.0,
+               269: 0.0,
+               261: 0.0,
+               266: 1198.0,
+               267: 0.0,
+               262: 2629.0,
+               258: 775.0,
+               257: 0.0,
+               263: 0.0,
+               259: 0.0,
+               264: 163.0,
+               250: 10326.0,
+               251: 0.0,
+               252: 1228.0,
+               253: 0.0,
+               254: 2769.0,
+               255: 0.0}
+
+        # smoke test for the repr
+        s = Series(ser)
+        result = s.value_counts()
+        str(result)
