@@ -6,11 +6,12 @@ import numpy as np
 import pandas as pd
 from pandas.core.api import Series, DataFrame, MultiIndex
 import pandas.util.testing as tm
+import pytest
 
 
 class TestIndexingSlow(object):
 
-    @tm.slow
+    @pytest.mark.slow
     def test_multiindex_get_loc(self):  # GH7724, GH2646
 
         with warnings.catch_warnings(record=True):
@@ -80,7 +81,7 @@ class TestIndexingSlow(object):
                     assert not mi.index.lexsort_depth < i
                     loop(mi, df, keys)
 
-    @tm.slow
+    @pytest.mark.slow
     def test_large_dataframe_indexing(self):
         # GH10692
         result = DataFrame({'x': range(10 ** 6)}, dtype='int64')
@@ -88,7 +89,7 @@ class TestIndexingSlow(object):
         expected = DataFrame({'x': range(10 ** 6 + 1)}, dtype='int64')
         tm.assert_frame_equal(result, expected)
 
-    @tm.slow
+    @pytest.mark.slow
     def test_large_mi_dataframe_indexing(self):
         # GH10645
         result = MultiIndex.from_arrays([range(10 ** 6), range(10 ** 6)])
