@@ -2,33 +2,25 @@
 from __future__ import print_function
 # pylint: disable-msg=W0612,E1101
 
-import nose
+from warnings import catch_warnings
 import re
+import operator
+import pytest
 
 from numpy.random import randn
 
-import operator
 import numpy as np
 
 from pandas.core.api import DataFrame, Panel
-from pandas.computation import expressions as expr
-from pandas import compat
+from pandas.core.computation import expressions as expr
+from pandas import compat, _np_version_under1p11, _np_version_under1p13
 from pandas.util.testing import (assert_almost_equal, assert_series_equal,
                                  assert_frame_equal, assert_panel_equal,
-                                 assert_panel4d_equal, slow)
-from pandas.formats.printing import pprint_thing
+                                 assert_panel4d_equal)
+from pandas.io.formats.printing import pprint_thing
 import pandas.util.testing as tm
 
 
-if not expr._USE_NUMEXPR:
-    try:
-        import numexpr  # noqa
-    except ImportError:
-        msg = "don't have"
-    else:
-        msg = "not using"
-    raise nose.SkipTest("{0} numexpr".format(msg))
-
 _frame = DataFrame(randn(10000, 4), columns=list('ABCD'), dtype='float64')
 _frame2 = DataFrame(randn(100, 4), columns=list('ABCD'), dtype='float64')
 _mixed = DataFrame({'A': _frame['A'].copy(),
@@ -41,26 +33,32 @@ _mixed2 = DataFrame({'A': _frame2['A'].copy(),
                      'D': _frame2['D'].astype('int32')})
 _integer = DataFrame(
     np.random.randint(1, 100,
-                      size=(10001, 4)), columns=list('ABCD'), dtype='int64')
+                      size=(10001, 4)),
+    columns=list('ABCD'), dtype='int64')
 _integer2 = DataFrame(np.random.randint(1, 100, size=(101, 4)),
                       columns=list('ABCD'), dtype='int64')
-_frame_panel = Panel(dict(ItemA=_frame.copy(), ItemB=(
-    _frame.copy() + 3), ItemC=_frame.copy(), ItemD=_frame.copy()))
-_frame2_panel = Panel(dict(ItemA=_frame2.copy(), ItemB=(_frame2.copy() + 3),
-                           ItemC=_frame2.copy(), ItemD=_frame2.copy()))
-_integer_panel = Panel(dict(ItemA=_integer, ItemB=(_integer + 34).astype(
-    'int64')))
-_integer2_panel = Panel(dict(ItemA=_integer2, ItemB=(_integer2 + 34).astype(
-    'int64')))
-_mixed_panel = Panel(dict(ItemA=_mixed, ItemB=(_mixed + 3)))
-_mixed2_panel = Panel(dict(ItemA=_mixed2, ItemB=(_mixed2 + 3)))
+
+with catch_warnings(record=True):
+    _frame_panel = Panel(dict(ItemA=_frame.copy(),
+                              ItemB=(_frame.copy() + 3),
+                              ItemC=_frame.copy(),
+                              ItemD=_frame.copy()))
+    _frame2_panel = Panel(dict(ItemA=_frame2.copy(),
+                               ItemB=(_frame2.copy() + 3),
+                               ItemC=_frame2.copy(),
+                               ItemD=_frame2.copy()))
+    _integer_panel = Panel(dict(ItemA=_integer,
+                                ItemB=(_integer + 34).astype('int64')))
+    _integer2_panel = Panel(dict(ItemA=_integer2,
+                                 ItemB=(_integer2 + 34).astype('int64')))
+    _mixed_panel = Panel(dict(ItemA=_mixed, ItemB=(_mixed + 3)))
+    _mixed2_panel = Panel(dict(ItemA=_mixed2, ItemB=(_mixed2 + 3)))
 
 
-class TestExpressions(tm.TestCase):
+@pytest.mark.skipif(not expr._USE_NUMEXPR, reason='not using numexpr')
+class TestExpressions(object):
 
-    _multiprocess_can_split_ = False
-
-    def setUp(self):
+    def setup_method(self, method):
 
         self.frame = _frame.copy()
         self.frame2 = _frame2.copy()
@@ -69,17 +67,17 @@ class TestExpressions(tm.TestCase):
         self.integer = _integer.copy()
         self._MIN_ELEMENTS = expr._MIN_ELEMENTS
 
-    def tearDown(self):
+    def teardown_method(self, method):
         expr._MIN_ELEMENTS = self._MIN_ELEMENTS
 
-    @nose.tools.nottest
-    def run_arithmetic_test(self, df, other, assert_func, check_dtype=False,
-                            test_flex=True):
+    def run_arithmetic(self, df, other, assert_func, check_dtype=False,
+                       test_flex=True):
         expr._MIN_ELEMENTS = 0
-        operations = ['add', 'sub', 'mul', 'mod', 'truediv', 'floordiv', 'pow']
+        operations = ['add', 'sub', 'mul', 'mod', 'truediv', 'floordiv']
         if not compat.PY3:
             operations.append('div')
         for arith in operations:
+
             operator_name = arith
             if arith == 'div':
                 operator_name = 'truediv'
@@ -92,6 +90,7 @@ class TestExpressions(tm.TestCase):
             expr.set_use_numexpr(False)
             expected = op(df, other)
             expr.set_use_numexpr(True)
+
             result = op(df, other)
             try:
                 if check_dtype:
@@ -103,15 +102,14 @@ class TestExpressions(tm.TestCase):
                 raise
 
     def test_integer_arithmetic(self):
-        self.run_arithmetic_test(self.integer, self.integer,
-                                 assert_frame_equal)
-        self.run_arithmetic_test(self.integer.iloc[:, 0],
-                                 self.integer.iloc[:, 0], assert_series_equal,
-                                 check_dtype=True)
+        self.run_arithmetic(self.integer, self.integer,
+                            assert_frame_equal)
+        self.run_arithmetic(self.integer.iloc[:, 0],
+                            self.integer.iloc[:, 0], assert_series_equal,
+                            check_dtype=True)
 
-    @nose.tools.nottest
-    def run_binary_test(self, df, other, assert_func, test_flex=False,
-                        numexpr_ops=set(['gt', 'lt', 'ge', 'le', 'eq', 'ne'])):
+    def run_binary(self, df, other, assert_func, test_flex=False,
+                   numexpr_ops=set(['gt', 'lt', 'ge', 'le', 'eq', 'ne'])):
         """
         tests solely that the result is the same whether or not numexpr is
         enabled.  Need to test whether the function does the correct thing
@@ -120,6 +118,7 @@ class TestExpressions(tm.TestCase):
         expr._MIN_ELEMENTS = 0
         expr.set_test_mode(True)
         operations = ['gt', 'lt', 'ge', 'le', 'eq', 'ne']
+
         for arith in operations:
             if test_flex:
                 op = lambda x, y: getattr(df, arith)(y)
@@ -145,46 +144,46 @@ class TestExpressions(tm.TestCase):
 
     def run_frame(self, df, other, binary_comp=None, run_binary=True,
                   **kwargs):
-        self.run_arithmetic_test(df, other, assert_frame_equal,
-                                 test_flex=False, **kwargs)
-        self.run_arithmetic_test(df, other, assert_frame_equal, test_flex=True,
-                                 **kwargs)
+        self.run_arithmetic(df, other, assert_frame_equal,
+                            test_flex=False, **kwargs)
+        self.run_arithmetic(df, other, assert_frame_equal, test_flex=True,
+                            **kwargs)
         if run_binary:
             if binary_comp is None:
                 expr.set_use_numexpr(False)
                 binary_comp = other + 1
                 expr.set_use_numexpr(True)
-            self.run_binary_test(df, binary_comp, assert_frame_equal,
-                                 test_flex=False, **kwargs)
-            self.run_binary_test(df, binary_comp, assert_frame_equal,
-                                 test_flex=True, **kwargs)
+            self.run_binary(df, binary_comp, assert_frame_equal,
+                            test_flex=False, **kwargs)
+            self.run_binary(df, binary_comp, assert_frame_equal,
+                            test_flex=True, **kwargs)
 
     def run_series(self, ser, other, binary_comp=None, **kwargs):
-        self.run_arithmetic_test(ser, other, assert_series_equal,
-                                 test_flex=False, **kwargs)
-        self.run_arithmetic_test(ser, other, assert_almost_equal,
-                                 test_flex=True, **kwargs)
+        self.run_arithmetic(ser, other, assert_series_equal,
+                            test_flex=False, **kwargs)
+        self.run_arithmetic(ser, other, assert_almost_equal,
+                            test_flex=True, **kwargs)
         # series doesn't uses vec_compare instead of numexpr...
         # if binary_comp is None:
         #     binary_comp = other + 1
-        # self.run_binary_test(ser, binary_comp, assert_frame_equal,
+        # self.run_binary(ser, binary_comp, assert_frame_equal,
         # test_flex=False, **kwargs)
-        # self.run_binary_test(ser, binary_comp, assert_frame_equal,
+        # self.run_binary(ser, binary_comp, assert_frame_equal,
         # test_flex=True, **kwargs)
 
     def run_panel(self, panel, other, binary_comp=None, run_binary=True,
                   assert_func=assert_panel_equal, **kwargs):
-        self.run_arithmetic_test(panel, other, assert_func, test_flex=False,
-                                 **kwargs)
-        self.run_arithmetic_test(panel, other, assert_func, test_flex=True,
-                                 **kwargs)
+        self.run_arithmetic(panel, other, assert_func, test_flex=False,
+                            **kwargs)
+        self.run_arithmetic(panel, other, assert_func, test_flex=True,
+                            **kwargs)
         if run_binary:
             if binary_comp is None:
                 binary_comp = other + 1
-            self.run_binary_test(panel, binary_comp, assert_func,
-                                 test_flex=False, **kwargs)
-            self.run_binary_test(panel, binary_comp, assert_func,
-                                 test_flex=True, **kwargs)
+            self.run_binary(panel, binary_comp, assert_func,
+                            test_flex=False, **kwargs)
+            self.run_binary(panel, binary_comp, assert_func,
+                            test_flex=True, **kwargs)
 
     def test_integer_arithmetic_frame(self):
         self.run_frame(self.integer, self.integer)
@@ -192,7 +191,7 @@ class TestExpressions(tm.TestCase):
     def test_integer_arithmetic_series(self):
         self.run_series(self.integer.iloc[:, 0], self.integer.iloc[:, 0])
 
-    @slow
+    @pytest.mark.slow
     def test_integer_panel(self):
         self.run_panel(_integer2_panel, np.random.randint(1, 100))
 
@@ -202,13 +201,13 @@ class TestExpressions(tm.TestCase):
     def test_float_arithmetic_series(self):
         self.run_series(self.frame2.iloc[:, 0], self.frame2.iloc[:, 0])
 
-    @slow
+    @pytest.mark.slow
     def test_float_panel(self):
         self.run_panel(_frame2_panel, np.random.randn() + 0.1, binary_comp=0.8)
 
-    @slow
+    @pytest.mark.slow
     def test_panel4d(self):
-        with tm.assert_produces_warning(FutureWarning, check_stacklevel=False):
+        with catch_warnings(record=True):
             self.run_panel(tm.makePanel4D(), np.random.randn() + 0.5,
                            assert_func=assert_panel4d_equal, binary_comp=3)
 
@@ -222,50 +221,50 @@ class TestExpressions(tm.TestCase):
         for col in self.mixed2.columns:
             self.run_series(self.mixed2[col], self.mixed2[col], binary_comp=4)
 
-    @slow
+    @pytest.mark.slow
     def test_mixed_panel(self):
         self.run_panel(_mixed2_panel, np.random.randint(1, 100),
                        binary_comp=-2)
 
     def test_float_arithemtic(self):
-        self.run_arithmetic_test(self.frame, self.frame, assert_frame_equal)
-        self.run_arithmetic_test(self.frame.iloc[:, 0], self.frame.iloc[:, 0],
-                                 assert_series_equal, check_dtype=True)
+        self.run_arithmetic(self.frame, self.frame, assert_frame_equal)
+        self.run_arithmetic(self.frame.iloc[:, 0], self.frame.iloc[:, 0],
+                            assert_series_equal, check_dtype=True)
 
     def test_mixed_arithmetic(self):
-        self.run_arithmetic_test(self.mixed, self.mixed, assert_frame_equal)
+        self.run_arithmetic(self.mixed, self.mixed, assert_frame_equal)
         for col in self.mixed.columns:
-            self.run_arithmetic_test(self.mixed[col], self.mixed[col],
-                                     assert_series_equal)
+            self.run_arithmetic(self.mixed[col], self.mixed[col],
+                                assert_series_equal)
 
     def test_integer_with_zeros(self):
         self.integer *= np.random.randint(0, 2, size=np.shape(self.integer))
-        self.run_arithmetic_test(self.integer, self.integer,
-                                 assert_frame_equal)
-        self.run_arithmetic_test(self.integer.iloc[:, 0],
-                                 self.integer.iloc[:, 0], assert_series_equal)
+        self.run_arithmetic(self.integer, self.integer,
+                            assert_frame_equal)
+        self.run_arithmetic(self.integer.iloc[:, 0],
+                            self.integer.iloc[:, 0], assert_series_equal)
 
     def test_invalid(self):
 
         # no op
         result = expr._can_use_numexpr(operator.add, None, self.frame,
                                        self.frame, 'evaluate')
-        self.assertFalse(result)
+        assert not result
 
         # mixed
         result = expr._can_use_numexpr(operator.add, '+', self.mixed,
                                        self.frame, 'evaluate')
-        self.assertFalse(result)
+        assert not result
 
         # min elements
         result = expr._can_use_numexpr(operator.add, '+', self.frame2,
                                        self.frame2, 'evaluate')
-        self.assertFalse(result)
+        assert not result
 
         # ok, we only check on first part of expression
         result = expr._can_use_numexpr(operator.add, '+', self.frame,
                                        self.frame2, 'evaluate')
-        self.assertTrue(result)
+        assert result
 
     def test_binary_ops(self):
         def testit():
@@ -275,6 +274,13 @@ class TestExpressions(tm.TestCase):
 
                 for op, op_str in [('add', '+'), ('sub', '-'), ('mul', '*'),
                                    ('div', '/'), ('pow', '**')]:
+
+                    # numpy >= 1.11 doesn't handle integers
+                    # raised to integer powers
+                    # https://github.com/pandas-dev/pandas/issues/15363
+                    if op == 'pow' and not _np_version_under1p11:
+                        continue
+
                     if op == 'div':
                         op = getattr(operator, 'truediv', None)
                     else:
@@ -282,7 +288,7 @@ class TestExpressions(tm.TestCase):
                     if op is not None:
                         result = expr._can_use_numexpr(op, op_str, f, f,
                                                        'evaluate')
-                        self.assertNotEqual(result, f._is_mixed_type)
+                        assert result != f._is_mixed_type
 
                         result = expr.evaluate(op, op_str, f, f,
                                                use_numexpr=True)
@@ -297,7 +303,7 @@ class TestExpressions(tm.TestCase):
 
                         result = expr._can_use_numexpr(op, op_str, f2, f2,
                                                        'evaluate')
-                        self.assertFalse(result)
+                        assert not result
 
         expr.set_use_numexpr(False)
         testit()
@@ -325,7 +331,7 @@ class TestExpressions(tm.TestCase):
 
                     result = expr._can_use_numexpr(op, op_str, f11, f12,
                                                    'evaluate')
-                    self.assertNotEqual(result, f11._is_mixed_type)
+                    assert result != f11._is_mixed_type
 
                     result = expr.evaluate(op, op_str, f11, f12,
                                            use_numexpr=True)
@@ -338,7 +344,7 @@ class TestExpressions(tm.TestCase):
 
                     result = expr._can_use_numexpr(op, op_str, f21, f22,
                                                    'evaluate')
-                    self.assertFalse(result)
+                    assert not result
 
         expr.set_use_numexpr(False)
         testit()
@@ -379,22 +385,22 @@ class TestExpressions(tm.TestCase):
                 f = getattr(operator, name)
                 err_msg = re.escape(msg % op)
 
-                with tm.assertRaisesRegexp(NotImplementedError, err_msg):
+                with tm.assert_raises_regex(NotImplementedError, err_msg):
                     f(df, df)
 
-                with tm.assertRaisesRegexp(NotImplementedError, err_msg):
+                with tm.assert_raises_regex(NotImplementedError, err_msg):
                     f(df.a, df.b)
 
-                with tm.assertRaisesRegexp(NotImplementedError, err_msg):
+                with tm.assert_raises_regex(NotImplementedError, err_msg):
                     f(df.a, True)
 
-                with tm.assertRaisesRegexp(NotImplementedError, err_msg):
+                with tm.assert_raises_regex(NotImplementedError, err_msg):
                     f(False, df.a)
 
-                with tm.assertRaisesRegexp(TypeError, err_msg):
+                with tm.assert_raises_regex(TypeError, err_msg):
                     f(False, df)
 
-                with tm.assertRaisesRegexp(TypeError, err_msg):
+                with tm.assert_raises_regex(TypeError, err_msg):
                     f(df, True)
 
     def test_bool_ops_warn_on_arithmetic(self):
@@ -409,6 +415,10 @@ class TestExpressions(tm.TestCase):
             f = getattr(operator, name)
             fe = getattr(operator, sub_funcs[subs[op]])
 
+            # >= 1.13.0 these are now TypeErrors
+            if op == '-' and not _np_version_under1p13:
+                continue
+
             with tm.use_numexpr(True, min_elements=5):
                 with tm.assert_produces_warning(check_stacklevel=False):
                     r = f(df, df)
@@ -439,9 +449,3 @@ class TestExpressions(tm.TestCase):
                     r = f(df, True)
                     e = fe(df, True)
                     tm.assert_frame_equal(r, e)
-
-
-if __name__ == '__main__':
-    import nose
-    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
-                   exit=False)
