@@ -1,22 +1,33 @@
-from datetime import datetime,timedelta
-from pandas.compat import range, long, zip
+# -*- coding: utf-8 -*-
+from datetime import timedelta
+from pandas.compat import long, zip
 from pandas import compat
 import re
 import warnings
 
 import numpy as np
 
+from pandas.core.dtypes.generic import ABCSeries
+from pandas.core.dtypes.common import (
+    is_period_arraylike,
+    is_timedelta64_dtype,
+    is_datetime64_dtype)
+
+import pandas.core.algorithms as algos
 from pandas.core.algorithms import unique
 from pandas.tseries.offsets import DateOffset
-from pandas.util.decorators import cache_readonly
+from pandas.util._decorators import cache_readonly, deprecate_kwarg
 import pandas.tseries.offsets as offsets
-import pandas.core.common as com
-import pandas.lib as lib
-import pandas.tslib as tslib
-import pandas._period as period
-from pandas.tslib import Timedelta
+
+from pandas._libs import lib, tslib
+from pandas._libs.tslib import Timedelta
+from pandas._libs.tslibs.frequencies import (  # noqa
+    get_freq_code, _base_and_stride, _period_str_to_code,
+    _INVALID_FREQ_ERROR, opattern, _lite_rule_alias, _dont_uppercase,
+    _period_code_map, _reverse_period_code_map)
 from pytz import AmbiguousTimeError
 
+
 class FreqGroup(object):
     FR_ANN = 1000
     FR_QTR = 2000
@@ -32,40 +43,72 @@ class FreqGroup(object):
     FR_NS = 12000
 
 
+RESO_NS = 0
+RESO_US = 1
+RESO_MS = 2
+RESO_SEC = 3
+RESO_MIN = 4
+RESO_HR = 5
+RESO_DAY = 6
+
+
 class Resolution(object):
 
-    # defined in period.pyx
-    # note that these are different from freq codes
-    RESO_US = period.US_RESO
-    RESO_MS = period.MS_RESO
-    RESO_SEC = period.S_RESO
-    RESO_MIN = period.T_RESO
-    RESO_HR = period.H_RESO
-    RESO_DAY = period.D_RESO
+    RESO_US = RESO_US
+    RESO_MS = RESO_MS
+    RESO_SEC = RESO_SEC
+    RESO_MIN = RESO_MIN
+    RESO_HR = RESO_HR
+    RESO_DAY = RESO_DAY
 
     _reso_str_map = {
-    RESO_US: 'microsecond',
-    RESO_MS: 'millisecond',
-    RESO_SEC: 'second',
-    RESO_MIN: 'minute',
-    RESO_HR: 'hour',
-    RESO_DAY: 'day'}
+        RESO_NS: 'nanosecond',
+        RESO_US: 'microsecond',
+        RESO_MS: 'millisecond',
+        RESO_SEC: 'second',
+        RESO_MIN: 'minute',
+        RESO_HR: 'hour',
+        RESO_DAY: 'day'
+    }
+
+    # factor to multiply a value by to convert it to the next finer grained
+    # resolution
+    _reso_mult_map = {
+        RESO_NS: None,
+        RESO_US: 1000,
+        RESO_MS: 1000,
+        RESO_SEC: 1000,
+        RESO_MIN: 60,
+        RESO_HR: 60,
+        RESO_DAY: 24
+    }
+
+    _reso_str_bump_map = {
+        'D': 'H',
+        'H': 'T',
+        'T': 'S',
+        'S': 'L',
+        'L': 'U',
+        'U': 'N',
+        'N': None
+    }
 
     _str_reso_map = dict([(v, k) for k, v in compat.iteritems(_reso_str_map)])
 
     _reso_freq_map = {
-    'year': 'A',
-    'quarter': 'Q',
-    'month': 'M',
-    'day': 'D',
-    'hour': 'H',
-    'minute': 'T',
-    'second': 'S',
-    'millisecond': 'L',
-    'microsecond': 'U',
-    'nanosecond': 'N'}
+        'year': 'A',
+        'quarter': 'Q',
+        'month': 'M',
+        'day': 'D',
+        'hour': 'H',
+        'minute': 'T',
+        'second': 'S',
+        'millisecond': 'L',
+        'microsecond': 'U',
+        'nanosecond': 'N'}
 
-    _freq_reso_map = dict([(v, k) for k, v in compat.iteritems(_reso_freq_map)])
+    _freq_reso_map = dict([(v, k)
+                           for k, v in compat.iteritems(_reso_freq_map)])
 
     @classmethod
     def get_str(cls, reso):
@@ -145,6 +188,47 @@ class Resolution(object):
         """
         return cls.get_reso(cls.get_str_from_freq(freq))
 
+    @classmethod
+    def get_stride_from_decimal(cls, value, freq):
+        """
+        Convert freq with decimal stride into a higher freq with integer stride
+
+        Parameters
+        ----------
+        value : integer or float
+        freq : string
+            Frequency string
+
+        Raises
+        ------
+        ValueError
+            If the float cannot be converted to an integer at any resolution.
+
+        Example
+        -------
+        >>> Resolution.get_stride_from_decimal(1.5, 'T')
+        (90, 'S')
+
+        >>> Resolution.get_stride_from_decimal(1.04, 'H')
+        (3744, 'S')
+
+        >>> Resolution.get_stride_from_decimal(1, 'D')
+        (1, 'D')
+        """
+
+        if np.isclose(value % 1, 0):
+            return int(value), freq
+        else:
+            start_reso = cls.get_reso_from_freq(freq)
+            if start_reso == 0:
+                raise ValueError(
+                    "Could not convert to integer offset at any resolution"
+                )
+
+            next_value = cls._reso_mult_map[start_reso] * value
+            next_name = cls._reso_str_bump_map[freq]
+            return cls.get_stride_from_decimal(next_value, next_name)
+
 
 def get_to_timestamp_base(base):
     """
@@ -218,58 +302,6 @@ def get_freq(freq):
     return freq
 
 
-def get_freq_code(freqstr):
-    """
-    Return freq str or tuple to freq code and stride (mult)
-
-    Parameters
-    ----------
-    freqstr : str or tuple
-
-    Returns
-    -------
-    return : tuple of base frequency code and stride (mult)
-
-    Example
-    -------
-    >>> get_freq_code('3D')
-    (6000, 3)
-
-    >>> get_freq_code('D')
-    (6000, 1)
-
-    >>> get_freq_code(('D', 3))
-    (6000, 3)
-    """
-    if isinstance(freqstr, DateOffset):
-        freqstr = (freqstr.rule_code, freqstr.n)
-
-    if isinstance(freqstr, tuple):
-        if (com.is_integer(freqstr[0]) and
-                com.is_integer(freqstr[1])):
-            # e.g., freqstr = (2000, 1)
-            return freqstr
-        else:
-            # e.g., freqstr = ('T', 5)
-            try:
-                code = _period_str_to_code(freqstr[0])
-                stride = freqstr[1]
-            except:
-                if com.is_integer(freqstr[1]):
-                    raise
-                code = _period_str_to_code(freqstr[1])
-                stride = freqstr[0]
-            return code, stride
-
-    if com.is_integer(freqstr):
-        return (freqstr, 1)
-
-    base, stride = _base_and_stride(freqstr)
-    code = _period_str_to_code(base)
-
-    return code, stride
-
-
 def _get_freq_str(base, mult=1):
     code = _reverse_period_code_map.get(base)
     if mult == 1:
@@ -277,17 +309,17 @@ def _get_freq_str(base, mult=1):
     return str(mult) + code
 
 
-#----------------------------------------------------------------------
+# ---------------------------------------------------------------------
 # Offset names ("time rules") and related functions
 
 
-from pandas.tseries.offsets import (Nano, Micro, Milli, Second, Minute, Hour,
+from pandas.tseries.offsets import (Nano, Micro, Milli, Second,  # noqa
+                                    Minute, Hour,
                                     Day, BDay, CDay, Week, MonthBegin,
                                     MonthEnd, BMonthBegin, BMonthEnd,
                                     QuarterBegin, QuarterEnd, BQuarterBegin,
                                     BQuarterEnd, YearBegin, YearEnd,
-                                    BYearBegin, BYearEnd, _make_offset
-                                    )
+                                    BYearBegin, BYearEnd, prefix_mapping)
 try:
     cday = CDay()
 except NotImplementedError:
@@ -319,83 +351,33 @@ _offset_to_period_map = {
     'Q': 'Q',
     'A': 'A',
     'W': 'W',
-    'M': 'M'
+    'M': 'M',
+    'Y': 'A',
+    'BY': 'A',
+    'YS': 'A',
+    'BYS': 'A',
 }
 
-need_suffix = ['QS', 'BQ', 'BQS', 'AS', 'BA', 'BAS']
+need_suffix = ['QS', 'BQ', 'BQS', 'YS', 'AS', 'BY', 'BA', 'BYS', 'BAS']
 for __prefix in need_suffix:
     for _m in tslib._MONTHS:
-        _offset_to_period_map['%s-%s' % (__prefix, _m)] = \
-            _offset_to_period_map[__prefix]
+        _alias = '{prefix}-{month}'.format(prefix=__prefix, month=_m)
+        _offset_to_period_map[_alias] = _offset_to_period_map[__prefix]
 for __prefix in ['A', 'Q']:
     for _m in tslib._MONTHS:
-        _alias = '%s-%s' % (__prefix, _m)
+        _alias = '{prefix}-{month}'.format(prefix=__prefix, month=_m)
         _offset_to_period_map[_alias] = _alias
 
 _days = ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN']
 for _d in _days:
-    _offset_to_period_map['W-%s' % _d] = 'W-%s' % _d
+    _alias = 'W-{day}'.format(day=_d)
+    _offset_to_period_map[_alias] = _alias
 
 
 def get_period_alias(offset_str):
     """ alias to closest period strings BQ->Q etc"""
     return _offset_to_period_map.get(offset_str, None)
 
-_rule_aliases = {
-    # Legacy rules that will continue to map to their original values
-    # essentially for the rest of time
-    'WEEKDAY': 'B',
-    'EOM': 'BM',
-    'W@MON': 'W-MON',
-    'W@TUE': 'W-TUE',
-    'W@WED': 'W-WED',
-    'W@THU': 'W-THU',
-    'W@FRI': 'W-FRI',
-    'W@SAT': 'W-SAT',
-    'W@SUN': 'W-SUN',
-    'Q@JAN': 'BQ-JAN',
-    'Q@FEB': 'BQ-FEB',
-    'Q@MAR': 'BQ-MAR',
-    'A@JAN': 'BA-JAN',
-    'A@FEB': 'BA-FEB',
-    'A@MAR': 'BA-MAR',
-    'A@APR': 'BA-APR',
-    'A@MAY': 'BA-MAY',
-    'A@JUN': 'BA-JUN',
-    'A@JUL': 'BA-JUL',
-    'A@AUG': 'BA-AUG',
-    'A@SEP': 'BA-SEP',
-    'A@OCT': 'BA-OCT',
-    'A@NOV': 'BA-NOV',
-    'A@DEC': 'BA-DEC',
-}
-
-_lite_rule_alias = {
-    'W': 'W-SUN',
-    'Q': 'Q-DEC',
-
-    'A': 'A-DEC',  # YearEnd(month=12),
-    'AS': 'AS-JAN',  # YearBegin(month=1),
-    'BA': 'BA-DEC',  # BYearEnd(month=12),
-    'BAS': 'BAS-JAN',  # BYearBegin(month=1),
-
-    'Min': 'T',
-    'min': 'T',
-    'ms': 'L',
-    'us': 'U'
-}
-
-#TODO: Can this be killed?
-for _i, _weekday in enumerate(['MON', 'TUE', 'WED', 'THU', 'FRI']):
-    for _iweek in range(4):
-        _name = 'WOM-%d%s' % (_iweek + 1, _weekday)
-        _rule_aliases[_name.replace('-', '@')] = _name
-
-# Note that _rule_aliases is not 1:1 (d[BA]==d[A@DEC]), and so traversal
-# order matters when constructing an inverse. we pick one. #2331
-# Used in get_legacy_offset_name
-_legacy_reverse_map = dict((v, k) for k, v in
-                           reversed(sorted(compat.iteritems(_rule_aliases))))
 
 _name_to_offset_map = {'days': Day(1),
                        'hours': Hour(1),
@@ -405,37 +387,72 @@ _name_to_offset_map = {'days': Day(1),
                        'microseconds': Micro(1),
                        'nanoseconds': Nano(1)}
 
-def to_offset(freqstr):
+
+@deprecate_kwarg(old_arg_name='freqstr', new_arg_name='freq')
+def to_offset(freq):
     """
-    Return DateOffset object from string representation or
-    Timedelta object
+    Return DateOffset object from string or tuple representation
+    or datetime.timedelta object
+
+    Parameters
+    ----------
+    freq : str, tuple, datetime.timedelta, DateOffset or None
+
+    Returns
+    -------
+    delta : DateOffset
+        None if freq is None
+
+    Raises
+    ------
+    ValueError
+        If freq is an invalid frequency
+
+    See Also
+    --------
+    pandas.DateOffset
 
     Examples
     --------
-    >>> to_offset('5Min')
-    Minute(5)
+    >>> to_offset('5min')
+    <5 * Minutes>
+
+    >>> to_offset('1D1H')
+    <25 * Hours>
+
+    >>> to_offset(('W', 2))
+    <2 * Weeks: weekday=6>
+
+    >>> to_offset((2, 'B'))
+    <2 * BusinessDays>
+
+    >>> to_offset(datetime.timedelta(days=1))
+    <Day>
+
+    >>> to_offset(Hour())
+    <Hour>
     """
-    if freqstr is None:
+    if freq is None:
         return None
 
-    if isinstance(freqstr, DateOffset):
-        return freqstr
+    if isinstance(freq, DateOffset):
+        return freq
 
-    if isinstance(freqstr, tuple):
-        name = freqstr[0]
-        stride = freqstr[1]
+    if isinstance(freq, tuple):
+        name = freq[0]
+        stride = freq[1]
         if isinstance(stride, compat.string_types):
             name, stride = stride, name
         name, _ = _base_and_stride(name)
         delta = get_offset(name) * stride
 
-    elif isinstance(freqstr, timedelta):
+    elif isinstance(freq, timedelta):
         delta = None
-        freqstr = Timedelta(freqstr)
+        freq = Timedelta(freq)
         try:
-            for name in freqstr.components._fields:
+            for name in freq.components._fields:
                 offset = _name_to_offset_map[name]
-                stride = getattr(freqstr.components, name)
+                stride = getattr(freq.components, name)
                 if stride != 0:
                     offset = stride * offset
                     if delta is None:
@@ -443,62 +460,45 @@ def to_offset(freqstr):
                     else:
                         delta = delta + offset
         except Exception:
-            raise ValueError("Could not evaluate %s" % freqstr)
+            raise ValueError(_INVALID_FREQ_ERROR.format(freq))
 
     else:
         delta = None
         stride_sign = None
         try:
-            for stride, name, _ in opattern.findall(freqstr):
-                offset = get_offset(name)
+            splitted = re.split(opattern, freq)
+            if splitted[-1] != '' and not splitted[-1].isspace():
+                # the last element must be blank
+                raise ValueError('last element must be blank')
+            for sep, stride, name in zip(splitted[0::4], splitted[1::4],
+                                         splitted[2::4]):
+                if sep != '' and not sep.isspace():
+                    raise ValueError('separator must be spaces')
+                prefix = _lite_rule_alias.get(name) or name
                 if stride_sign is None:
                     stride_sign = -1 if stride.startswith('-') else 1
                 if not stride:
                     stride = 1
+                if prefix in Resolution._reso_str_bump_map.keys():
+                    stride, name = Resolution.get_stride_from_decimal(
+                        float(stride), prefix
+                    )
                 stride = int(stride)
+                offset = get_offset(name)
                 offset = offset * int(np.fabs(stride) * stride_sign)
                 if delta is None:
                     delta = offset
                 else:
                     delta = delta + offset
         except Exception:
-            raise ValueError("Could not evaluate %s" % freqstr)
+            raise ValueError(_INVALID_FREQ_ERROR.format(freq))
 
     if delta is None:
-        raise ValueError('Unable to understand %s as a frequency' % freqstr)
+        raise ValueError(_INVALID_FREQ_ERROR.format(freq))
 
     return delta
 
 
-# hack to handle WOM-1MON
-opattern = re.compile(r'([\-]?\d*)\s*([A-Za-z]+([\-@][\dA-Za-z\-]+)?)')
-
-
-def _base_and_stride(freqstr):
-    """
-    Return base freq and stride info from string representation
-
-    Examples
-    --------
-    _freq_and_stride('5Min') -> 'Min', 5
-    """
-    groups = opattern.match(freqstr)
-
-    if not groups:
-        raise ValueError("Could not evaluate %s" % freqstr)
-
-    stride = groups.group(1)
-
-    if len(stride):
-        stride = int(stride)
-    else:
-        stride = 1
-
-    base = groups.group(2)
-
-    return (base, stride)
-
-
 def get_base_alias(freqstr):
     """
     Returns the base frequency alias, e.g., '5D' -> 'D'
@@ -506,12 +506,6 @@ def get_base_alias(freqstr):
     return _base_and_stride(freqstr)[0]
 
 
-_dont_uppercase = set(('MS', 'ms'))
-
-
-_LEGACY_FREQ_WARNING = 'Freq "{0}" is deprecated, use "{1}" as alternative.'
-
-
 def get_offset(name):
     """
     Return DateOffset object associated with rule name
@@ -522,296 +516,44 @@ def get_offset(name):
     """
     if name not in _dont_uppercase:
         name = name.upper()
-
-        if name in _rule_aliases:
-            new = _rule_aliases[name]
-            warnings.warn(_LEGACY_FREQ_WARNING.format(name, new),
-                          FutureWarning, stacklevel=2)
-            name = new
-        elif name.lower() in _rule_aliases:
-            new = _rule_aliases[name.lower()]
-            warnings.warn(_LEGACY_FREQ_WARNING.format(name, new),
-                          FutureWarning, stacklevel=2)
-            name = new
-
         name = _lite_rule_alias.get(name, name)
         name = _lite_rule_alias.get(name.lower(), name)
-
     else:
-        if name in _rule_aliases:
-            new = _rule_aliases[name]
-            warnings.warn(_LEGACY_FREQ_WARNING.format(name, new),
-                          FutureWarning, stacklevel=2)
-            name = new
         name = _lite_rule_alias.get(name, name)
 
     if name not in _offset_map:
         try:
-            # generate and cache offset
-            offset = _make_offset(name)
+            split = name.split('-')
+            klass = prefix_mapping[split[0]]
+            # handles case where there's no suffix (and will TypeError if too
+            # many '-')
+            offset = klass._from_name(*split[1:])
         except (ValueError, TypeError, KeyError):
             # bad prefix or suffix
-            raise ValueError('Bad rule name requested: %s.' % name)
+            raise ValueError(_INVALID_FREQ_ERROR.format(name))
+        # cache
         _offset_map[name] = offset
-    return _offset_map[name]
+    # do not return cache because it's mutable
+    return _offset_map[name].copy()
 
 
 getOffset = get_offset
 
 
-def get_offset_name(offset):
-    """
-    Return rule name associated with a DateOffset object
-
-    Examples
-    --------
-    get_offset_name(BMonthEnd(1)) --> 'EOM'
-    """
-    if offset is None:
-        raise ValueError("Offset can't be none!")
-    # Hack because this is what it did before...
-    if isinstance(offset, BDay):
-        if offset.n != 1:
-            raise ValueError('Bad rule given: %s.' % 'BusinessDays')
-        else:
-            return offset.rule_code
-    try:
-        return offset.freqstr
-    except AttributeError:
-        # Bad offset, give useful error.
-        raise ValueError('Bad rule given: %s.' % offset)
-
-
-def get_legacy_offset_name(offset):
-    """
-    Return the pre pandas 0.8.0 name for the date offset
-    """
-
-    # This only used in test_timeseries_legacy.py
-
-    name = offset.name
-    return _legacy_reverse_map.get(name, name)
-
 def get_standard_freq(freq):
     """
     Return the standardized frequency string
     """
-    if freq is None:
-        return None
 
-    if isinstance(freq, DateOffset):
-        return freq.rule_code
+    msg = ("get_standard_freq is deprecated. Use to_offset(freq).rule_code "
+           "instead.")
+    warnings.warn(msg, FutureWarning, stacklevel=2)
+    return to_offset(freq).rule_code
 
-    code, stride = get_freq_code(freq)
-    return _get_freq_str(code, stride)
 
-#----------------------------------------------------------------------
+# ---------------------------------------------------------------------
 # Period codes
 
-# period frequency constants corresponding to scikits timeseries
-# originals
-_period_code_map = {
-    # Annual freqs with various fiscal year ends.
-    # eg, 2005 for A-FEB runs Mar 1, 2004 to Feb 28, 2005
-    "A-DEC": 1000,  # Annual - December year end
-    "A-JAN": 1001,  # Annual - January year end
-    "A-FEB": 1002,  # Annual - February year end
-    "A-MAR": 1003,  # Annual - March year end
-    "A-APR": 1004,  # Annual - April year end
-    "A-MAY": 1005,  # Annual - May year end
-    "A-JUN": 1006,  # Annual - June year end
-    "A-JUL": 1007,  # Annual - July year end
-    "A-AUG": 1008,  # Annual - August year end
-    "A-SEP": 1009,  # Annual - September year end
-    "A-OCT": 1010,  # Annual - October year end
-    "A-NOV": 1011,  # Annual - November year end
-
-    # Quarterly frequencies with various fiscal year ends.
-    # eg, Q42005 for Q-OCT runs Aug 1, 2005 to Oct 31, 2005
-    "Q-DEC": 2000,    # Quarterly - December year end
-    "Q-JAN": 2001,    # Quarterly - January year end
-    "Q-FEB": 2002,    # Quarterly - February year end
-    "Q-MAR": 2003,    # Quarterly - March year end
-    "Q-APR": 2004,    # Quarterly - April year end
-    "Q-MAY": 2005,    # Quarterly - May year end
-    "Q-JUN": 2006,    # Quarterly - June year end
-    "Q-JUL": 2007,    # Quarterly - July year end
-    "Q-AUG": 2008,    # Quarterly - August year end
-    "Q-SEP": 2009,    # Quarterly - September year end
-    "Q-OCT": 2010,    # Quarterly - October year end
-    "Q-NOV": 2011,    # Quarterly - November year end
-
-    "M": 3000,        # Monthly
-
-    "W-SUN": 4000,    # Weekly - Sunday end of week
-    "W-MON": 4001,    # Weekly - Monday end of week
-    "W-TUE": 4002,    # Weekly - Tuesday end of week
-    "W-WED": 4003,    # Weekly - Wednesday end of week
-    "W-THU": 4004,    # Weekly - Thursday end of week
-    "W-FRI": 4005,    # Weekly - Friday end of week
-    "W-SAT": 4006,    # Weekly - Saturday end of week
-
-    "B": 5000,        # Business days
-    "D": 6000,        # Daily
-    "H": 7000,        # Hourly
-    "T": 8000,        # Minutely
-    "S": 9000,        # Secondly
-    "L": 10000,       # Millisecondly
-    "U": 11000,       # Microsecondly
-    "N": 12000,       # Nanosecondly
-}
-
-_reverse_period_code_map = {}
-for _k, _v in compat.iteritems(_period_code_map):
-    _reverse_period_code_map[_v] = _k
-
-# Additional aliases
-_period_code_map.update({
-    "Q": 2000,  # Quarterly - December year end (default quarterly)
-    "A": 1000,  # Annual
-    "W": 4000,  # Weekly
-})
-
-
-def _period_alias_dictionary():
-    """
-    Build freq alias dictionary to support freqs from original c_dates.c file
-    of the scikits.timeseries library.
-    """
-    alias_dict = {}
-
-    M_aliases = ["M", "MTH", "MONTH", "MONTHLY"]
-    B_aliases = ["B", "BUS", "BUSINESS", "BUSINESSLY", 'WEEKDAY']
-    D_aliases = ["D", "DAY", "DLY", "DAILY"]
-    H_aliases = ["H", "HR", "HOUR", "HRLY", "HOURLY"]
-    T_aliases = ["T", "MIN", "MINUTE", "MINUTELY"]
-    S_aliases = ["S", "SEC", "SECOND", "SECONDLY"]
-    L_aliases = ["L", "ms", "MILLISECOND", "MILLISECONDLY"]
-    U_aliases = ["U", "US", "MICROSECOND", "MICROSECONDLY"]
-    N_aliases = ["N", "NS", "NANOSECOND", "NANOSECONDLY"]
-
-    for k in M_aliases:
-        alias_dict[k] = 'M'
-
-    for k in B_aliases:
-        alias_dict[k] = 'B'
-
-    for k in D_aliases:
-        alias_dict[k] = 'D'
-
-    for k in H_aliases:
-        alias_dict[k] = 'H'
-
-    for k in T_aliases:
-        alias_dict[k] = 'Min'
-
-    for k in S_aliases:
-        alias_dict[k] = 'S'
-
-    for k in L_aliases:
-        alias_dict[k] = 'L'
-
-    for k in U_aliases:
-        alias_dict[k] = 'U'
-
-    for k in N_aliases:
-        alias_dict[k] = 'N'
-
-    A_prefixes = ["A", "Y", "ANN", "ANNUAL", "ANNUALLY", "YR", "YEAR",
-                  "YEARLY"]
-
-    Q_prefixes = ["Q", "QTR", "QUARTER", "QUARTERLY", "Q-E",
-                  "QTR-E", "QUARTER-E", "QUARTERLY-E"]
-
-    month_names = [
-        ["DEC", "DECEMBER"],
-        ["JAN", "JANUARY"],
-        ["FEB", "FEBRUARY"],
-        ["MAR", "MARCH"],
-        ["APR", "APRIL"],
-        ["MAY", "MAY"],
-        ["JUN", "JUNE"],
-        ["JUL", "JULY"],
-        ["AUG", "AUGUST"],
-        ["SEP", "SEPTEMBER"],
-        ["OCT", "OCTOBER"],
-        ["NOV", "NOVEMBER"]]
-
-    seps = ["@", "-"]
-
-    for k in A_prefixes:
-        alias_dict[k] = 'A'
-        for m_tup in month_names:
-            for sep in seps:
-                m1, m2 = m_tup
-                alias_dict[k + sep + m1] = 'A-' + m1
-                alias_dict[k + sep + m2] = 'A-' + m1
-
-    for k in Q_prefixes:
-        alias_dict[k] = 'Q'
-        for m_tup in month_names:
-            for sep in seps:
-                m1, m2 = m_tup
-                alias_dict[k + sep + m1] = 'Q-' + m1
-                alias_dict[k + sep + m2] = 'Q-' + m1
-
-    W_prefixes = ["W", "WK", "WEEK", "WEEKLY"]
-
-    day_names = [
-        ["SUN", "SUNDAY"],
-        ["MON", "MONDAY"],
-        ["TUE", "TUESDAY"],
-        ["WED", "WEDNESDAY"],
-        ["THU", "THURSDAY"],
-        ["FRI", "FRIDAY"],
-        ["SAT", "SATURDAY"]]
-
-    for k in W_prefixes:
-        alias_dict[k] = 'W'
-        for d_tup in day_names:
-            for sep in ["@", "-"]:
-                d1, d2 = d_tup
-                alias_dict[k + sep + d1] = 'W-' + d1
-                alias_dict[k + sep + d2] = 'W-' + d1
-
-    return alias_dict
-
-
-_period_alias_dict = _period_alias_dictionary()
-
-
-def _period_str_to_code(freqstr):
-    # hack
-    if freqstr in _rule_aliases:
-        new = _rule_aliases[freqstr]
-        warnings.warn(_LEGACY_FREQ_WARNING.format(freqstr, new),
-                      FutureWarning, stacklevel=3)
-        freqstr = new
-    freqstr = _lite_rule_alias.get(freqstr, freqstr)
-
-    if freqstr not in _dont_uppercase:
-        lower = freqstr.lower()
-        if lower in _rule_aliases:
-            new = _rule_aliases[lower]
-            warnings.warn(_LEGACY_FREQ_WARNING.format(lower, new),
-                          FutureWarning, stacklevel=3)
-            freqstr = new
-        freqstr = _lite_rule_alias.get(lower, freqstr)
-
-    try:
-        if freqstr not in _dont_uppercase:
-            freqstr = freqstr.upper()
-        return _period_code_map[freqstr]
-    except KeyError:
-        try:
-            alias = _period_alias_dict[freqstr]
-            warnings.warn(_LEGACY_FREQ_WARNING.format(freqstr, alias),
-                          FutureWarning, stacklevel=3)
-        except KeyError:
-            raise ValueError("Unknown freqstr: %s" % freqstr)
-
-        return _period_code_map[alias]
-
 
 def infer_freq(index, warn=True):
     """
@@ -821,7 +563,7 @@ def infer_freq(index, warn=True):
     Parameters
     ----------
     index : DatetimeIndex or TimedeltaIndex
-            if passed a Series will use the values of the series (NOT THE INDEX)
+      if passed a Series will use the values of the series (NOT THE INDEX)
     warn : boolean, default True
 
     Returns
@@ -833,13 +575,16 @@ def infer_freq(index, warn=True):
     """
     import pandas as pd
 
-    if isinstance(index, com.ABCSeries):
+    if isinstance(index, ABCSeries):
         values = index._values
-        if not (com.is_datetime64_dtype(values) or com.is_timedelta64_dtype(values) or values.dtype == object):
-            raise TypeError("cannot infer freq from a non-convertible dtype on a Series of {0}".format(index.dtype))
+        if not (is_datetime64_dtype(values) or
+                is_timedelta64_dtype(values) or
+                values.dtype == object):
+            raise TypeError("cannot infer freq from a non-convertible dtype "
+                            "on a Series of {dtype}".format(dtype=index.dtype))
         index = values
 
-    if com.is_period_arraylike(index):
+    if is_period_arraylike(index):
         raise TypeError("PeriodIndex given. Check the `freq` attribute "
                         "instead of using infer_freq.")
     elif isinstance(index, pd.TimedeltaIndex):
@@ -848,7 +593,8 @@ def infer_freq(index, warn=True):
 
     if isinstance(index, pd.Index) and not isinstance(index, pd.DatetimeIndex):
         if isinstance(index, (pd.Int64Index, pd.Float64Index)):
-            raise TypeError("cannot infer freq from a non-convertible index type {0}".format(type(index)))
+            raise TypeError("cannot infer freq from a non-convertible index "
+                            "type {type}".format(type=type(index)))
         index = index.values
 
     if not isinstance(index, pd.DatetimeIndex):
@@ -860,6 +606,7 @@ def infer_freq(index, warn=True):
     inferer = _FrequencyInferer(index, warn=warn)
     return inferer.get_freq()
 
+
 _ONE_MICRO = long(1000)
 _ONE_MILLI = _ONE_MICRO * 1000
 _ONE_SECOND = _ONE_MILLI * 1000
@@ -879,7 +626,7 @@ class _FrequencyInferer(object):
 
         # This moves the values, which are implicitly in UTC, to the
         # the timezone so they are in local time
-        if hasattr(index,'tz'):
+        if hasattr(index, 'tz'):
             if index.tz is not None:
                 self.values = tslib.tz_convert(self.values, 'UTC', index.tz)
 
@@ -1014,15 +761,17 @@ class _FrequencyInferer(object):
         if annual_rule:
             nyears = self.ydiffs[0]
             month = _month_aliases[self.rep_stamp.month]
-            return _maybe_add_count('%s-%s' % (annual_rule, month), nyears)
+            alias = '{prefix}-{month}'.format(prefix=annual_rule, month=month)
+            return _maybe_add_count(alias, nyears)
 
         quarterly_rule = self._get_quarterly_rule()
         if quarterly_rule:
             nquarters = self.mdiffs[0] / 3
             mod_dict = {0: 12, 2: 11, 1: 10}
             month = _month_aliases[mod_dict[self.rep_stamp.month % 3]]
-            return _maybe_add_count('%s-%s' % (quarterly_rule, month),
-                                    nquarters)
+            alias = '{prefix}-{month}'.format(prefix=quarterly_rule,
+                                              month=month)
+            return _maybe_add_count(alias, nquarters)
 
         monthly_rule = self._get_monthly_rule()
         if monthly_rule:
@@ -1032,13 +781,12 @@ class _FrequencyInferer(object):
             days = self.deltas[0] / _ONE_DAY
             if days % 7 == 0:
                 # Weekly
-                alias = _weekday_rule_aliases[self.rep_stamp.weekday()]
-                return _maybe_add_count('W-%s' % alias, days / 7)
+                day = _weekday_rule_aliases[self.rep_stamp.weekday()]
+                return _maybe_add_count('W-{day}'.format(day=day), days / 7)
             else:
                 return _maybe_add_count('D', days)
 
-        # Business daily. Maybe
-        if self.day_deltas == [1, 3]:
+        if self._is_business_daily():
             return 'B'
 
         wom_rule = self._get_wom_rule()
@@ -1074,11 +822,24 @@ class _FrequencyInferer(object):
         return {'cs': 'MS', 'bs': 'BMS',
                 'ce': 'M', 'be': 'BM'}.get(pos_check)
 
+    def _is_business_daily(self):
+        # quick check: cannot be business daily
+        if self.day_deltas != [1, 3]:
+            return False
+
+        # probably business daily, but need to confirm
+        first_weekday = self.index[0].weekday()
+        shifts = np.diff(self.index.asi8)
+        shifts = np.floor_divide(shifts, _ONE_DAY)
+        weekdays = np.mod(first_weekday + np.cumsum(shifts), 7)
+        return np.all(((weekdays == 0) & (shifts == 3)) |
+                      ((weekdays > 0) & (weekdays <= 4) & (shifts == 1)))
+
     def _get_wom_rule(self):
-#         wdiffs = unique(np.diff(self.index.week))
-        #We also need -47, -49, -48 to catch index spanning year boundary
-#         if not lib.ismember(wdiffs, set([4, 5, -47, -49, -48])).all():
-#             return None
+        #         wdiffs = unique(np.diff(self.index.week))
+        # We also need -47, -49, -48 to catch index spanning year boundary
+        #     if not lib.ismember(wdiffs, set([4, 5, -47, -49, -48])).all():
+        #         return None
 
         weekdays = unique(self.index.weekday)
         if len(weekdays) > 1:
@@ -1094,9 +855,8 @@ class _FrequencyInferer(object):
         week = week_of_months[0] + 1
         wd = _weekday_rule_aliases[weekdays[0]]
 
-        return 'WOM-%d%s' % (week, wd)
+        return 'WOM-{week}{weekday}'.format(week=week, weekday=wd)
 
-import pandas.core.algorithms as algos
 
 class _TimedeltaFrequencyInferer(_FrequencyInferer):
 
@@ -1105,19 +865,40 @@ class _TimedeltaFrequencyInferer(_FrequencyInferer):
             days = self.deltas[0] / _ONE_DAY
             if days % 7 == 0:
                 # Weekly
-                alias = _weekday_rule_aliases[self.rep_stamp.weekday()]
-                return _maybe_add_count('W-%s' % alias, days / 7)
+                wd = _weekday_rule_aliases[self.rep_stamp.weekday()]
+                alias = 'W-{weekday}'.format(weekday=wd)
+                return _maybe_add_count(alias, days / 7)
             else:
                 return _maybe_add_count('D', days)
 
 
 def _maybe_add_count(base, count):
     if count != 1:
-        return '%d%s' % (count, base)
+        return '{count}{base}'.format(count=int(count), base=base)
     else:
         return base
 
 
+def _maybe_coerce_freq(code):
+    """ we might need to coerce a code to a rule_code
+    and uppercase it
+
+    Parameters
+    ----------
+    source : string
+        Frequency converting from
+
+    Returns
+    -------
+    string code
+    """
+
+    assert code is not None
+    if isinstance(code, offsets.DateOffset):
+        code = code.rule_code
+    return code.upper()
+
+
 def is_subperiod(source, target):
     """
     Returns True if downsampling is possible between source and target
@@ -1134,14 +915,12 @@ def is_subperiod(source, target):
     -------
     is_subperiod : boolean
     """
-    if isinstance(source, offsets.DateOffset):
-        source = source.rule_code
 
-    if isinstance(target, offsets.DateOffset):
-        target = target.rule_code
+    if target is None or source is None:
+        return False
+    source = _maybe_coerce_freq(source)
+    target = _maybe_coerce_freq(target)
 
-    target = target.upper()
-    source = source.upper()
     if _is_annual(target):
         if _is_quarterly(source):
             return _quarter_months_conform(_get_rule_month(source),
@@ -1189,14 +968,11 @@ def is_superperiod(source, target):
     -------
     is_superperiod : boolean
     """
-    if isinstance(source, offsets.DateOffset):
-        source = source.rule_code
+    if target is None or source is None:
+        return False
+    source = _maybe_coerce_freq(source)
+    target = _maybe_coerce_freq(target)
 
-    if isinstance(target, offsets.DateOffset):
-        target = target.rule_code
-
-    target = target.upper()
-    source = source.upper()
     if _is_annual(source):
         if _is_annual(target):
             return _get_rule_month(source) == _get_rule_month(target)
@@ -1268,5 +1044,6 @@ _month_numbers = tslib._MONTH_NUMBERS
 _month_aliases = tslib._MONTH_ALIASES
 _weekday_rule_aliases = dict((k, v) for k, v in enumerate(DAYS))
 
+
 def _is_multiple(us, mult):
     return us % mult == 0
