@@ -1,4 +1,5 @@
 from pandas import *
+from pandas.compat import range
 try:
     import pandas.core.internals as internals
     reload(internals)
@@ -17,7 +18,7 @@ def horribly_unconsolidated():
 
     df = DataMatrix(index=index)
 
-    for i in xrange(K):
+    for i in range(K):
         df[i] = float(K)
 
     return df
@@ -25,13 +26,13 @@ def horribly_unconsolidated():
 
 def bench_reindex_index(df, it=100):
     new_idx = np.arange(0, N, 2)
-    for i in xrange(it):
+    for i in range(it):
         df.reindex(new_idx)
 
 
 def bench_reindex_columns(df, it=100):
     new_cols = np.arange(0, K, 2)
-    for i in xrange(it):
+    for i in range(it):
         df.reindex(columns=new_cols)
 
 
@@ -39,7 +40,7 @@ def bench_join_index(df, it=10):
     left = df.reindex(index=np.arange(0, N, 2),
                       columns=np.arange(K // 2))
     right = df.reindex(columns=np.arange(K // 2 + 1, K))
-    for i in xrange(it):
+    for i in range(it):
         joined = left.join(right)
 
 if __name__ == '__main__':
